/*
Copyright 2020 Rancher Labs.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"time"

	v1 "github.com/rancher/rio/pkg/apis/admin.rio.cattle.io/v1"
	scheme "github.com/rancher/rio/pkg/generated/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// PublicDomainsGetter has a method to return a PublicDomainInterface.
// A group's client should implement this interface.
type PublicDomainsGetter interface {
	PublicDomains() PublicDomainInterface
}

// PublicDomainInterface has methods to work with PublicDomain resources.
type PublicDomainInterface interface {
	Create(*v1.PublicDomain) (*v1.PublicDomain, error)
	Update(*v1.PublicDomain) (*v1.PublicDomain, error)
	UpdateStatus(*v1.PublicDomain) (*v1.PublicDomain, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.PublicDomain, error)
	List(opts metav1.ListOptions) (*v1.PublicDomainList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.PublicDomain, err error)
	PublicDomainExpansion
}

// publicDomains implements PublicDomainInterface
type publicDomains struct {
	client rest.Interface
}

// newPublicDomains returns a PublicDomains
func newPublicDomains(c *AdminV1Client) *publicDomains {
	return &publicDomains{
		client: c.RESTClient(),
	}
}

// Get takes name of the publicDomain, and returns the corresponding publicDomain object, and an error if there is any.
func (c *publicDomains) Get(name string, options metav1.GetOptions) (result *v1.PublicDomain, err error) {
	result = &v1.PublicDomain{}
	err = c.client.Get().
		Resource("publicdomains").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of PublicDomains that match those selectors.
func (c *publicDomains) List(opts metav1.ListOptions) (result *v1.PublicDomainList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.PublicDomainList{}
	err = c.client.Get().
		Resource("publicdomains").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested publicDomains.
func (c *publicDomains) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("publicdomains").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a publicDomain and creates it.  Returns the server's representation of the publicDomain, and an error, if there is any.
func (c *publicDomains) Create(publicDomain *v1.PublicDomain) (result *v1.PublicDomain, err error) {
	result = &v1.PublicDomain{}
	err = c.client.Post().
		Resource("publicdomains").
		Body(publicDomain).
		Do().
		Into(result)
	return
}

// Update takes the representation of a publicDomain and updates it. Returns the server's representation of the publicDomain, and an error, if there is any.
func (c *publicDomains) Update(publicDomain *v1.PublicDomain) (result *v1.PublicDomain, err error) {
	result = &v1.PublicDomain{}
	err = c.client.Put().
		Resource("publicdomains").
		Name(publicDomain.Name).
		Body(publicDomain).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *publicDomains) UpdateStatus(publicDomain *v1.PublicDomain) (result *v1.PublicDomain, err error) {
	result = &v1.PublicDomain{}
	err = c.client.Put().
		Resource("publicdomains").
		Name(publicDomain.Name).
		SubResource("status").
		Body(publicDomain).
		Do().
		Into(result)
	return
}

// Delete takes name of the publicDomain and deletes it. Returns an error if one occurs.
func (c *publicDomains) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Resource("publicdomains").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *publicDomains) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("publicdomains").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched publicDomain.
func (c *publicDomains) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.PublicDomain, err error) {
	result = &v1.PublicDomain{}
	err = c.client.Patch(pt).
		Resource("publicdomains").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
