/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"time"

	v1 "github.com/rancher/gitwatcher/pkg/apis/gitwatcher.cattle.io/v1"
	scheme "github.com/rancher/gitwatcher/pkg/generated/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// GitWatchersGetter has a method to return a GitWatcherInterface.
// A group's client should implement this interface.
type GitWatchersGetter interface {
	GitWatchers(namespace string) GitWatcherInterface
}

// GitWatcherInterface has methods to work with GitWatcher resources.
type GitWatcherInterface interface {
	Create(*v1.GitWatcher) (*v1.GitWatcher, error)
	Update(*v1.GitWatcher) (*v1.GitWatcher, error)
	UpdateStatus(*v1.GitWatcher) (*v1.GitWatcher, error)
	Delete(name string, options *metav1.DeleteOptions) error
	DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error
	Get(name string, options metav1.GetOptions) (*v1.GitWatcher, error)
	List(opts metav1.ListOptions) (*v1.GitWatcherList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.GitWatcher, err error)
	GitWatcherExpansion
}

// gitWatchers implements GitWatcherInterface
type gitWatchers struct {
	client rest.Interface
	ns     string
}

// newGitWatchers returns a GitWatchers
func newGitWatchers(c *GitwatcherV1Client, namespace string) *gitWatchers {
	return &gitWatchers{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the gitWatcher, and returns the corresponding gitWatcher object, and an error if there is any.
func (c *gitWatchers) Get(name string, options metav1.GetOptions) (result *v1.GitWatcher, err error) {
	result = &v1.GitWatcher{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("gitwatchers").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of GitWatchers that match those selectors.
func (c *gitWatchers) List(opts metav1.ListOptions) (result *v1.GitWatcherList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.GitWatcherList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("gitwatchers").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested gitWatchers.
func (c *gitWatchers) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("gitwatchers").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a gitWatcher and creates it.  Returns the server's representation of the gitWatcher, and an error, if there is any.
func (c *gitWatchers) Create(gitWatcher *v1.GitWatcher) (result *v1.GitWatcher, err error) {
	result = &v1.GitWatcher{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("gitwatchers").
		Body(gitWatcher).
		Do().
		Into(result)
	return
}

// Update takes the representation of a gitWatcher and updates it. Returns the server's representation of the gitWatcher, and an error, if there is any.
func (c *gitWatchers) Update(gitWatcher *v1.GitWatcher) (result *v1.GitWatcher, err error) {
	result = &v1.GitWatcher{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("gitwatchers").
		Name(gitWatcher.Name).
		Body(gitWatcher).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *gitWatchers) UpdateStatus(gitWatcher *v1.GitWatcher) (result *v1.GitWatcher, err error) {
	result = &v1.GitWatcher{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("gitwatchers").
		Name(gitWatcher.Name).
		SubResource("status").
		Body(gitWatcher).
		Do().
		Into(result)
	return
}

// Delete takes name of the gitWatcher and deletes it. Returns an error if one occurs.
func (c *gitWatchers) Delete(name string, options *metav1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("gitwatchers").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *gitWatchers) DeleteCollection(options *metav1.DeleteOptions, listOptions metav1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("gitwatchers").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched gitWatcher.
func (c *gitWatchers) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.GitWatcher, err error) {
	result = &v1.GitWatcher{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("gitwatchers").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
