/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1alpha2

import (
	"context"
	"time"

	v1alpha2 "github.com/jetstack/cert-manager/pkg/apis/certmanager/v1alpha2"
	clientset "github.com/rancher/wrangler-api/pkg/generated/clientset/versioned/typed/certmanager/v1alpha2"
	informers "github.com/rancher/wrangler-api/pkg/generated/informers/externalversions/certmanager/v1alpha2"
	listers "github.com/rancher/wrangler-api/pkg/generated/listers/certmanager/v1alpha2"
	"github.com/rancher/wrangler/pkg/apply"
	"github.com/rancher/wrangler/pkg/condition"
	"github.com/rancher/wrangler/pkg/generic"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type CertificateHandler func(string, *v1alpha2.Certificate) (*v1alpha2.Certificate, error)

type CertificateController interface {
	generic.ControllerMeta
	CertificateClient

	OnChange(ctx context.Context, name string, sync CertificateHandler)
	OnRemove(ctx context.Context, name string, sync CertificateHandler)
	Enqueue(namespace, name string)
	EnqueueAfter(namespace, name string, duration time.Duration)

	Cache() CertificateCache
}

type CertificateClient interface {
	Create(*v1alpha2.Certificate) (*v1alpha2.Certificate, error)
	Update(*v1alpha2.Certificate) (*v1alpha2.Certificate, error)
	UpdateStatus(*v1alpha2.Certificate) (*v1alpha2.Certificate, error)
	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1alpha2.Certificate, error)
	List(namespace string, opts metav1.ListOptions) (*v1alpha2.CertificateList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha2.Certificate, err error)
}

type CertificateCache interface {
	Get(namespace, name string) (*v1alpha2.Certificate, error)
	List(namespace string, selector labels.Selector) ([]*v1alpha2.Certificate, error)

	AddIndexer(indexName string, indexer CertificateIndexer)
	GetByIndex(indexName, key string) ([]*v1alpha2.Certificate, error)
}

type CertificateIndexer func(obj *v1alpha2.Certificate) ([]string, error)

type certificateController struct {
	controllerManager *generic.ControllerManager
	clientGetter      clientset.CertificatesGetter
	informer          informers.CertificateInformer
	gvk               schema.GroupVersionKind
}

func NewCertificateController(gvk schema.GroupVersionKind, controllerManager *generic.ControllerManager, clientGetter clientset.CertificatesGetter, informer informers.CertificateInformer) CertificateController {
	return &certificateController{
		controllerManager: controllerManager,
		clientGetter:      clientGetter,
		informer:          informer,
		gvk:               gvk,
	}
}

func FromCertificateHandlerToHandler(sync CertificateHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1alpha2.Certificate
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1alpha2.Certificate))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *certificateController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1alpha2.Certificate))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateCertificateDeepCopyOnChange(client CertificateClient, obj *v1alpha2.Certificate, handler func(obj *v1alpha2.Certificate) (*v1alpha2.Certificate, error)) (*v1alpha2.Certificate, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *certificateController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, handler)
}

func (c *certificateController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), handler)
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, removeHandler)
}

func (c *certificateController) OnChange(ctx context.Context, name string, sync CertificateHandler) {
	c.AddGenericHandler(ctx, name, FromCertificateHandlerToHandler(sync))
}

func (c *certificateController) OnRemove(ctx context.Context, name string, sync CertificateHandler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), FromCertificateHandlerToHandler(sync))
	c.AddGenericHandler(ctx, name, removeHandler)
}

func (c *certificateController) Enqueue(namespace, name string) {
	c.controllerManager.Enqueue(c.gvk, c.informer.Informer(), namespace, name)
}

func (c *certificateController) EnqueueAfter(namespace, name string, duration time.Duration) {
	c.controllerManager.EnqueueAfter(c.gvk, c.informer.Informer(), namespace, name, duration)
}

func (c *certificateController) Informer() cache.SharedIndexInformer {
	return c.informer.Informer()
}

func (c *certificateController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *certificateController) Cache() CertificateCache {
	return &certificateCache{
		lister:  c.informer.Lister(),
		indexer: c.informer.Informer().GetIndexer(),
	}
}

func (c *certificateController) Create(obj *v1alpha2.Certificate) (*v1alpha2.Certificate, error) {
	return c.clientGetter.Certificates(obj.Namespace).Create(obj)
}

func (c *certificateController) Update(obj *v1alpha2.Certificate) (*v1alpha2.Certificate, error) {
	return c.clientGetter.Certificates(obj.Namespace).Update(obj)
}

func (c *certificateController) UpdateStatus(obj *v1alpha2.Certificate) (*v1alpha2.Certificate, error) {
	return c.clientGetter.Certificates(obj.Namespace).UpdateStatus(obj)
}

func (c *certificateController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	return c.clientGetter.Certificates(namespace).Delete(name, options)
}

func (c *certificateController) Get(namespace, name string, options metav1.GetOptions) (*v1alpha2.Certificate, error) {
	return c.clientGetter.Certificates(namespace).Get(name, options)
}

func (c *certificateController) List(namespace string, opts metav1.ListOptions) (*v1alpha2.CertificateList, error) {
	return c.clientGetter.Certificates(namespace).List(opts)
}

func (c *certificateController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.clientGetter.Certificates(namespace).Watch(opts)
}

func (c *certificateController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha2.Certificate, err error) {
	return c.clientGetter.Certificates(namespace).Patch(name, pt, data, subresources...)
}

type certificateCache struct {
	lister  listers.CertificateLister
	indexer cache.Indexer
}

func (c *certificateCache) Get(namespace, name string) (*v1alpha2.Certificate, error) {
	return c.lister.Certificates(namespace).Get(name)
}

func (c *certificateCache) List(namespace string, selector labels.Selector) ([]*v1alpha2.Certificate, error) {
	return c.lister.Certificates(namespace).List(selector)
}

func (c *certificateCache) AddIndexer(indexName string, indexer CertificateIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1alpha2.Certificate))
		},
	}))
}

func (c *certificateCache) GetByIndex(indexName, key string) (result []*v1alpha2.Certificate, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	for _, obj := range objs {
		result = append(result, obj.(*v1alpha2.Certificate))
	}
	return result, nil
}

type CertificateStatusHandler func(obj *v1alpha2.Certificate, status v1alpha2.CertificateStatus) (v1alpha2.CertificateStatus, error)

type CertificateGeneratingHandler func(obj *v1alpha2.Certificate, status v1alpha2.CertificateStatus) ([]runtime.Object, v1alpha2.CertificateStatus, error)

func RegisterCertificateStatusHandler(ctx context.Context, controller CertificateController, condition condition.Cond, name string, handler CertificateStatusHandler) {
	statusHandler := &certificateStatusHandler{
		client:    controller,
		condition: condition,
		handler:   handler,
	}
	controller.AddGenericHandler(ctx, name, FromCertificateHandlerToHandler(statusHandler.sync))
}

func RegisterCertificateGeneratingHandler(ctx context.Context, controller CertificateController, apply apply.Apply,
	condition condition.Cond, name string, handler CertificateGeneratingHandler, opts *generic.GeneratingHandlerOptions) {
	statusHandler := &certificateGeneratingHandler{
		CertificateGeneratingHandler: handler,
		apply:                        apply,
		name:                         name,
		gvk:                          controller.GroupVersionKind(),
	}
	if opts != nil {
		statusHandler.opts = *opts
	}
	RegisterCertificateStatusHandler(ctx, controller, condition, name, statusHandler.Handle)
}

type certificateStatusHandler struct {
	client    CertificateClient
	condition condition.Cond
	handler   CertificateStatusHandler
}

func (a *certificateStatusHandler) sync(key string, obj *v1alpha2.Certificate) (*v1alpha2.Certificate, error) {
	if obj == nil {
		return obj, nil
	}

	status := obj.Status
	obj = obj.DeepCopy()
	newStatus, err := a.handler(obj, obj.Status)
	if err != nil {
		// Revert to old status on error
		newStatus = *status.DeepCopy()
	}

	if a.condition != "" {
		if errors.IsConflict(err) {
			a.condition.SetError(obj, "", nil)
		} else {
			a.condition.SetError(obj, "", err)
		}
	}
	if !equality.Semantic.DeepEqual(status, newStatus) {
		var newErr error
		obj.Status = newStatus
		obj, newErr = a.client.UpdateStatus(obj)
		if err == nil {
			err = newErr
		}
	}
	return obj, err
}

type certificateGeneratingHandler struct {
	CertificateGeneratingHandler
	apply apply.Apply
	opts  generic.GeneratingHandlerOptions
	gvk   schema.GroupVersionKind
	name  string
}

func (a *certificateGeneratingHandler) Handle(obj *v1alpha2.Certificate, status v1alpha2.CertificateStatus) (v1alpha2.CertificateStatus, error) {
	objs, newStatus, err := a.CertificateGeneratingHandler(obj, status)
	if err != nil {
		return newStatus, err
	}

	apply := a.apply

	if !a.opts.DynamicLookup {
		apply = apply.WithStrictCaching()
	}

	if !a.opts.AllowCrossNamespace && !a.opts.AllowClusterScoped {
		apply = apply.WithSetOwnerReference(true, false).
			WithDefaultNamespace(obj.GetNamespace()).
			WithListerNamespace(obj.GetNamespace())
	}

	if !a.opts.AllowClusterScoped {
		apply = apply.WithRestrictClusterScoped()
	}

	return newStatus, apply.
		WithOwner(obj).
		WithSetID(a.name).
		ApplyObjects(objs...)
}
